from plotly import tools
from plotly.offline.offline import _plot_html

import cufflinks as cf
import numpy as np
import pandas as pd
import plotly.graph_objs as go
import colorlover as cl

# local
from .episem import episem, lastepiday


cf.set_config_file(theme='white')


def ethio_ts(df=pd.DataFrame, scale_id=int, year=int):
    cols = [
        'Testes positivos',
        'Influenza A',
        'Influenza B',
        'VSR',
        'Adenovirus',
        'Parainfluenza 1',
        'Parainfluenza 2',
        'Parainfluenza 3',
    ]
    trace = []

    if scale_id == 2:
        ytitle = 'Casos'
    else:
        ytitle = 'Incidência'

    trace.append(
        go.Scatter(
            x=df['epiweek'],
            y=df[cols[0]],
            name=cols[0],
            mode='lines',
        )
    )

    for i, col in enumerate(cols[1:]):
        trace.append(
            go.Scatter(
                x=df['epiweek'],
                y=df[col],
                name=ytitle,
                mode='lines',
                showlegend=False
            )
        )

    nrows = len(cols)
    fig = tools.make_subplots(
        rows=nrows,
        cols=1,
        print_grid=False,
        subplot_titles=('Situação dos exames',
                        'Influenza A',
                        'Influenza B',
                        'VSR',
                        'Adenovirus',
                        'Parainfluenza 1',
                        'Parainfluenza 2',
                        'Parainfluenza 3'))

    for i in range(1, (nrows + 1)):
        fig.append_trace(trace[i - 1], i, 1)

    # Add extra lines to first subplot:
    extra_cols = ['Testes negativos',
                  'Casos sem teste laboratorial',
                  'Casos aguardando resultado',
                  'Casos sem informação laboratorial']
    for i, col in enumerate(extra_cols):
        fig.append_trace(
            go.Scatter(
                x=df['epiweek'],
                y=df[col],
                name=col,
                mode='lines',
                line=dict(color=cl.scales['12']['qual']['Paired'][-i])
            ), 1, 1)

    # X-axis title and range:
    lastepiweek = int(episem(lastepiday(year), out='W'))
    extra_cols.extend(['Testes positivos'])
    if scale_id == 2:
        ymax = [
            max(5*np.ceil(df[extra_cols].max().max()/5), 5),
            max(5*np.ceil(df[cols[1:]].max().max()/5), 5)
        ]
    else:
        ymax = [
            df[extra_cols].max().max(),
            df[cols[1:]].max().max()
        ]

    for i in range(1, (nrows + 1)):
        xaxis = 'xaxis%s' % i
        yaxis = 'yaxis%s' % i
        fig['layout'][xaxis].update(range=[1, lastepiweek])
        fig['layout'][yaxis].update(range=[0, ymax[min(1, i-1)]], rangemode='nonnegative')

    fig['layout']['xaxis%s' % nrows].update(title='Semana epidemiológica',
                                            zeroline=True, showline=True)

    i = int(nrows/2)
    fig['layout']['yaxis%s' % i].update(title=ytitle)

    fig['layout'].update(margin={'t': 50, 'l': 50})

    return _plot_html(
        figure_or_data=fig, config={}, validate=True,
        default_width='100%', default_height=1600, global_requirejs=''
    )[0]


def opportunities_boxplot(df: pd.DataFrame, week: int=None):
    """

    :param df:
    :param week:
    :return:

    """
    title_param = {
        'name': df.territory_name.unique()[0],
        'week': ''
    }

    if week not in [0, None]:
        title_param['week'] = 'até a semana epidemiológica %s ' % week

    df_plot = df.iloc[:, :-1].dropna(how='all', axis=1)

    # Set ymax to higher upper fence:
    q1 = df_plot.quantile(.25, axis=0)
    q3 = df_plot.quantile(.75, axis=0)
    ymax = (q3 + 1 * (q3 - q1)).max()

    # Set bottom and right margin to avoid xaxis labels beeing cut off
    figure = df_plot.iplot(
        kind='box',
        boxpoints=False,
        margin={'b': 130,
                't': 50,
                'r': 120,
                'l': 50
        },
        showlegend=False,
        yTitle='Dias',
        asFigure=True
    )

    figure['layout']['yaxis1'].update(range=[0, ymax])
    figure['layout']['xaxis1'].update(tickangle=30)

    return _plot_html(
        figure_or_data=figure, config={}, validate=True,
        default_width='100%', default_height=500, global_requirejs=''
    )[0]



from flask import render_template, Flask, Response
# local
from ..settings import APP_AVAILABLE
from .calc_flu_alert import (
    apply_filter_alert_by_epiweek,
    calc_alert_rank_whole_year
)
from .charts import ethio_ts, opportunities_boxplot
from .flu_data import FluDB
from .utils import cross_domain, calc_last_epiweek

import numpy as np
import os
import pandas as pd


app = Flask(
    __name__,
    template_folder='../templates',
    static_folder='../static'
)

fluDB = FluDB()


def compose_data_url(variables: str):
    """

    :param variables:
    :return:
    """
    url_var = {
        'data': '/data/<string:view_name>/<int:dataset_id>/<int:scale_id>',
        'year': '<int:year>',
        'epiweek': '<int:epiweek>',
        'territory_type_id': '<int:territory_type_id>',
        'territory_id': '<int:territory_id>',
        'territory_name': '<string:territory_name>',
    }

    url = [
        url_var[v] if v in url_var else v
        for v in ['data'] + variables.split('/')
    ]

    return '/'.join(url)


@app.errorhandler(404)
def page_not_found(e):
    return render_template('404.html'), 404


@app.errorhandler(403)
def page_forbidden(e):
    return render_template('403.html'), 403


@app.errorhandler(500)
def page_error_server(e):
    return render_template('500.html'), 500


@app.route("/super-header")
def super_header():
    return render_template("super-header.html")


@app.route("/")
def index():
    """

    :return:
    """
    if not APP_AVAILABLE:
        return render_template("unavailable.html")

    # read data to get the list of available years
    df = fluDB.read_data(
        table_name='current_estimated_values',
        dataset_id=1, scale_id=1, territory_id=0
    )

    # Here the code should receive the user-requested year.
    # By default should be the current or latest available
    list_of_years = list(set(df.epiyear))

    epiyear = df.epiyear.max()
    epiweek = df.epiweek[df.epiyear == epiyear].max()

    last_week_years = {
        y: calc_last_epiweek(y) for y in list_of_years
    }

    return render_template(
        "index.html",
        current_epiweek=epiweek,
        list_of_years=sorted(list_of_years, reverse=True),
        last_year=epiyear,
        last_week_years=last_week_years
    )


@app.route("/help")
def app_help():
    """

    :return:
    """
    return render_template("help.html")


@app.route(compose_data_url('year/territory_type_id'))
def get_data(
    view_name: str, dataset_id: int, scale_id: str, year: int,
    territory_type_id: int
):
    """
    :param view_name:
    :param dataset_id:
    :param scale_id:
    :param year:
    :param territory_type_id:
    :return:
    """
    df = fluDB.get_data(
        dataset_id=dataset_id, scale_id=scale_id, year=year,
        territory_type_id=territory_type_id, show_historical_weeks=False
    )
    return apply_filter_alert_by_epiweek(
        df, view_name
    ).to_json(orient='records')


@app.route(compose_data_url('year/epiweek/weekly-incidence-curve'))
@app.route(compose_data_url(
    'year/epiweek/territory_name/weekly-incidence-curve')
)
def data__weekly_incidence_curve(
    view_name: str, dataset_id: int, scale_id: int, year: int, epiweek: int,
    territory_name: str='Brasil'
):
    """
    :param view_name:
    :param dataset_id:
    :param scale_id:
    :param year:
    :param territory_name:
    :return:
    """
    if not year > 0:
        return '[]'

    ks = [
        'epiweek', 'typical_low', 'typical_median', 'typical_high',
        'value', 'pre_epidemic_threshold', 'high_threshold',
        'very_high_threshold'
    ]

    territory_id = fluDB.get_territory_id_from_name(territory_name)

    df = fluDB.get_data(
        dataset_id=dataset_id, scale_id=scale_id, year=year, week=epiweek,
        show_historical_weeks=True, territory_id=territory_id
    )

    try:
        ks += ['estimated_cases', 'ci_lower', 'ci_upper']

        k = 'estimated_cases'
        ks.pop(ks.index(k))
        ks.insert(ks.index('value') + 1, k)
    except:
        pass

    try:
        min_week = int(df.loc[df['situation_id'] == 4, 'epiweek'].min())
        mask = df['epiweek'] >= min_week

        df['incomplete_data'] = None
        df.loc[mask, 'incomplete_data'] = df.loc[mask, 'ci_upper']

        ks += ['incomplete_data']
    except:
        pass

    if len(df[~pd.isna(df.estimated_cases)]) > 0:
        est_min_week =  df.epiweek[~pd.isna(df.estimated_cases)].min()
        if est_min_week > 1:
            df.loc[(df.epiweek == est_min_week-1), 'estimated_cases'] = df.value[df.epiweek == est_min_week-1]

    # cheating: using a new field corredor_muito_alto just for plotting
    # cheating: using a new field corredor_muito_alto just for plotting
    df['typical_very_high'] = df.typical_high + df.typical_median + df.typical_low
    df['typical_very_high'] = df[[
            'very_high_threshold', 'ci_upper', 'value', 'typical_very_high'
        ]].max().max() * 1.1
    df['typical_very_high'] = df.typical_very_high - (df.typical_high + df.typical_median + df.typical_low)
    # change keys' order
    ks.insert(ks.index('typical_high') + 1, 'typical_very_high')

    return df[ks].to_csv(index=False, na_rep='null')


@app.route(compose_data_url('year/levels'))
@app.route(compose_data_url('year/epiweek/levels'))
@app.route(compose_data_url('year/epiweek/territory_name/levels'))
def data__incidence_levels(
    view_name: str, dataset_id: int, scale_id: int, year: int,
    epiweek: int=None, territory_name: str='Brasil'
):
    """
    When epiweek==None, the system will assume the whole year view.
    When state_name==None, the system will assume state_name=='Brasil'

    :param view_name:
    :param dataset_id:
    :param scale_id:
    :param year:
    :param epiweek:
    :param territory_name:
    :return:
    """
    if not year > 0:
        return '[]'

    territory_id = fluDB.get_territory_id_from_name(territory_name)

    df = fluDB.get_data(
        dataset_id=dataset_id, scale_id=scale_id, year=year,
        territory_id=territory_id, week=epiweek
    )

    if epiweek is not None and epiweek > 0:
        ks = [
            'low_level', 'epidemic_level',
            'high_level', 'very_high_level'
        ]

        df[ks] *= 100
        df[ks] = df[ks].round(2)

        ks += ['situation_id', 'regular_seasons']

        return df[ks].to_json(orient='records')

    ks = [
        'low_level', 'epidemic_level',
        'high_level', 'very_high_level'
    ]

    # prepare data for the whole year
    df = apply_filter_alert_by_epiweek(df=df, view_name=view_name)

    low_level = df[df.alert == 1].count().low_level
    epidemic_level = df[df.alert == 2].count().epidemic_level
    high_level = df[df.alert == 3].count().high_level
    very_high_level = df[df.alert == 4].count().very_high_level

    if scale_id == 1:
        total_level = low_level + epidemic_level + high_level + very_high_level

        low_level = np.round((low_level/total_level)*100, 2)
        epidemic_level = np.round((epidemic_level/total_level)*100, 2)
        high_level = np.round((high_level/total_level)*100, 2)
        very_high_level = np.round((very_high_level/total_level)*100, 2)

    se = pd.Series({
        'low_level': low_level,
        'epidemic_level': epidemic_level,
        'high_level': high_level,
        'very_high_level': very_high_level
    })

    regular_seasons = df.loc[0, 'regular_seasons']

    rank = calc_alert_rank_whole_year(se)

    se.l0 = 0
    se.l1 = 0
    se.l2 = 0
    se.l3 = 0
    se['l%s' % (rank-1)] = 1
    se['situation'] = ''

    df = pd.DataFrame(se).T
    df['regular_seasons'] = regular_seasons

    return df.to_json(orient='records')


@app.route(compose_data_url('year/data-table'))
@app.route(compose_data_url('year/epiweek/data-table'))
@app.route(compose_data_url('year/epiweek/territory_type_id/data-table'))
@app.route(
    compose_data_url(
        'year/epiweek/territory_type_id' +
        '/territory_name/data-table'
    )
)
def data__data_table(
    view_name: str, dataset_id: str, scale_id: str, year: int, epiweek: int=None,
    territory_type_id: str=None, territory_name: str=None
):
    """
    1. Total number of cases in the selected year for eac
       State + same data for the Country
    2. Number of cases in the selected week for each
       State + same data for the Country
    3. Total number of cases in the selected year for selected State
    4. Number of cases in the selected week for selected State.

    :param view_name:
    :param dataset_id:
    :param scale_id:
    :param year:
    :param epiweek:
    :param territory_type_id:
    :param territory_name:
    :return:

    """
    if not year > 0:
        return '{"data": []}'

    ks = [
        'territory_name',
        'epiweek',
        'situation_name',
        'value'
    ]

    if territory_name is not None:
        territory_id = fluDB.get_territory_id_from_name(territory_name)
    else:
        territory_id = None

    df = fluDB.get_data(
        dataset_id=dataset_id, scale_id=scale_id, year=year, week=epiweek,
        territory_id=territory_id
    )

    if territory_type_id == 1:  # state
        mask = ~(df.territory_type_name.isin(['Regional', 'Região']))
    elif territory_type_id == 2:  # regional
        mask = ~(df.territory_type_name.isin(['Estado', 'Região']))
    else:  # region
        mask = ~(df.territory_type_name.isin(['Estado', 'Regional']))

    df = df[mask]

    # for a whole year view
    if not epiweek:
        df = fluDB.group_data_by_season(df, season=year)

    # order by type
    df = df.assign(type_unit=1)

    try:
        df.loc[df.uf == 'BR', 'type_unit'] = 0
    except:
        pass

    df.sort_values(
        by=['type_unit', 'territory_name', 'epiyear', 'epiweek'],
        inplace=True
    )
    df.reset_index(drop=True, inplace=True)
    df.drop('type_unit', axis=1, inplace=True)

    # add more information into srag field
    if df.shape[0]:
        if epiweek:
            k = ['estimated_cases', 'ci_lower', 'ci_upper', 'situation_id']
            df.value = df[k].apply(
                lambda row: fluDB.report_incidence(
                    row['estimated_cases'],
                    row['situation_id'],
                    row['ci_lower'], row['ci_upper']
                ), axis=1)
        else:
            df.value = df[['value', 'situation_id']].apply(
                lambda row: fluDB.report_incidence(
                    row['value'], row['situation_id']
                ), axis=1)

        # change situation value by a informative text
        situation_dict = {
            'stable': 'Dado estável. Sujeito a pequenas alterações.',
            'estimated': 'Estimado. Sujeito a alterações.',
            'unknown': 'Dados incompletos. Sujeito a grandes alterações.',
            'incomplete': 'Dados incompletos. Sujeito a grandes alterações.'
        }

        df.situation_name = df.situation_name.map(
            lambda x: situation_dict[x] if x else ''
        )

    return '{"data": %s}' % df[ks].round({
        'value': 2
    }).to_json(orient='records')


@app.route(compose_data_url('year/age-distribution'))
@app.route(compose_data_url('year/epiweek/age-distribution'))
@app.route(compose_data_url('year/epiweek/territory_name/age-distribution'))
@cross_domain(origin='*')
def data__age_distribution(
    view_name: str, dataset_id: str, scale_id: str, year: int,
    epiweek: int=None, territory_name: str=None
):
    """

    :param view_name:
    :param dataset_id:
    :param scale_id:
    :param year:
    :param epiweek: 0 == all weeks
    :param territory_name:
    :return:
    """
    if not year > 0:
        return '[]'

    if not territory_name:
        territory_name = 'Brasil'

    territory_id = fluDB.get_territory_id_from_name(territory_name)

    df = pd.DataFrame(
        fluDB.get_data_age_sex(
            dataset_id=dataset_id, scale_id=scale_id,
            year=year, week=epiweek, territory_id=territory_id
        )
    ).round(2)

    # TODO: rename the data on the front-end side
    df.rename(index={
        'years_5_9': '5-9 anos',
        'years_10_19': '10-19 anos',
        'years_20_29': '20-29 anos',
        'years_30_39': '30-39 anos',
        'years_40_49': '40-49 anos',
        'years_50_59': '50-59 anos',
        'years_60_or_more': '60+ anos'
    }, inplace=True)
    if scale_id == 1:
        df.rename(index={'years_0_4': '0-4 anos'}, inplace=True)
    else:
        df.rename(index={'years_lt_2': '< 2 anos', 'years_2_4': '2-4 anos'}, inplace=True)

    # the replace is used when there is no data in the df
    return ('index' + df.to_csv()).replace('""', '')


@app.route(compose_data_url('year/etiological-agents'))
@app.route(compose_data_url('year/epiweek/etiological-agents'))
@app.route(compose_data_url('year/epiweek/territory_name/etiological-agents'))
def etiological_agents(
    view_name: str, dataset_id: str, scale_id: str, year: int,
    epiweek: int=None, territory_name: str=None
):
    """

    :param view_name:
    :param dataset_id:
    :param scale_id:
    :param year:
    :param epiweek:
    :param territory_name:
    :return:
    """
    territory_id = fluDB.get_territory_id_from_name(territory_name)

    # Etiological panel fixed to cases scale, not incidence.
    scale_id = 2

    df = FluDB().get_etiological_data(
        dataset_id=dataset_id, scale_id=scale_id, year=year,
        week=epiweek, territory_id=territory_id
    )

    return Response(ethio_ts(df, scale_id=scale_id, year=year))


@app.route(compose_data_url('year/opportunities-boxplot'))
@app.route(compose_data_url('year/epiweek/opportunities-boxplot'))
@app.route(compose_data_url('year/epiweek/territory_name/opportunities-boxplot'))
def opportunities_boxplot_view(
    view_name: str, dataset_id: str, scale_id: str, year: int,
    epiweek: int=None, territory_name: str=None
):
    """

    :param view_name:
    :param dataset_id:
    :param scale_id:
    :param year:
    :param epiweek:
    :param territory_name:
    :return:
    """
    territory = fluDB.get_territory_from_name(territory_name)
    territory_id = int(territory['id'])
    territory_type_id = int(territory['territory_type_id'])

    df = FluDB().get_opportunities(
        dataset_id=dataset_id, scale_id=scale_id, year=year,
        week=epiweek, territory_id=territory_id,
        territory_type_id=territory_type_id
    )

    return Response(opportunities_boxplot(df, week=epiweek))


@app.route('/static/data/br-states.json')
def static_data_br_states_view():
    """

    :return:
    """
    path_states_json = os.path.join(
        os.path.dirname(os.path.dirname(os.path.abspath(__file__))),
        'data', 'br-states.json'
    )
    with open(path_states_json) as f:
        resp = Response(f.read())
        resp.default_mimetype = 'plain/json'
        return resp



from fludashboard.libs.views import app
from fludashboard import settings


def startup():
    """

    :return:
    """
    app.run(
        host=settings.APP_HOST,
        port=settings.APP_PORT,
        debug=settings.DEBUG
    )

    return app


if __name__ == "__main__":
    startup()



import os
import yaml

home_path = os.path.expanduser("~")
settings_path = os.path.join(home_path, '.flu.yaml')

WSGI_NUM_WORKERS = None
WSGI_HOST = None
WSGI_PORT = None
WSGI_FLU_LOG = None

APP_HOST = None
APP_PORT = None
APP_AVAILABLE = True

DEBUG = None
DATABASE = {
    'NAME': None,
    'USER': None,
    'PASSWORD': None,
    'HOST': None,
    'PORT': None
}

# create yaml file
if not os.path.exists(settings_path):
    settings_yaml = {
        'WSGI_NUM_WORKERS': '4',
        'WSGI_HOST': '0.0.0.0',
        'WSGI_PORT': '8000',
        'WSGI_FLU_LOG': '.flu.log',
        'APP_HOST': '0.0.0.0',
        'APP_PORT': '5000',
        'APP_AVAILABLE': True,
        'DEBUG': False,
        'DATABASE': {
            'NAME': '<DATABASE_HERE>',
            'USER': '<DATABASE_USER_HERE>',
            'PASSWORD': '<DATABASE_PASSWORD_HERE>',
            'HOST': '<DATABASE_HOST_HERE>',
            'PORT': '<DATABASE_PORT_HERE>'
        }
    }

    with open(os.path.join(settings_path), 'w') as f:
        yaml.dump(settings_yaml, f, default_flow_style=False)

    raise Exception('Please configure your settings file (%s)' % settings_path  )

# load yaml file
with open(os.path.join(settings_path), 'r') as f:
    globals().update(yaml.load(f))

PATH = os.path.dirname(os.path.abspath(__file__))



from ..settings import DATABASE, PATH

import glob
import os
import pandas as pd
import sqlalchemy as sqla
import argparse
from argparse import RawDescriptionHelpFormatter

dataset_id = {
    'srag': 1,
    'sragflu': 2,
    'obitoflu': 3,
}

# ### 1.2 Scale

scale_id = {
    'incidência': 1,
    'casos': 2
}

# ### 1.3 Situation

situation_id = {
    'unknown': 1,
    'estimated': 2,
    'stable': 3,
    'incomplete': 4
}

# ### 1.4 Territory Type

territory_type_id = {
    'Estado': 1,
    'Regional': 2,
    'Região': 3,
    'País': 4
}

# ### 1.5 Region id conversion


region_id = {
    'BR': 0,
    'RegN': 1001,
    'RegL': 1002,
    'RegC': 1003,
    'RegS': 1004,
    'RegNI': 9999,
    'N': 1,
    'NE': 2,
    'SE': 3,
    'S': 4,
    'CO': 5,
    'RNI': 9
}

# ### Territory Table

df_territory = pd.DataFrame([
    {'id': 11, 'initials': 'RO', 'name': 'Rondônia',
     'territory_type_id': 1},
    {'id': 12, 'initials': 'AC', 'name': 'Acre', 'territory_type_id': 1},
    {'id': 13, 'initials': 'AM', 'name': 'Amazonas',
     'territory_type_id': 1},
    {'id': 14, 'initials': 'RR', 'name': 'Roraima',
     'territory_type_id': 1},
    {'id': 15, 'initials': 'PA', 'name': 'Pará', 'territory_type_id': 1},
    {'id': 16, 'initials': 'AP', 'name': 'Amapá', 'territory_type_id': 1},
    {'id': 17, 'initials': 'TO', 'name': 'Tocantins',
     'territory_type_id': 1},
    {'id': 21, 'initials': 'MA', 'name': 'Maranhão',
     'territory_type_id': 1},
    {'id': 22, 'initials': 'PI', 'name': 'Piauí', 'territory_type_id': 1},
    {'id': 23, 'initials': 'CE', 'name': 'Ceará', 'territory_type_id': 1},
    {'id': 24, 'initials': 'RN', 'name': 'Rio Grande do Norte',
     'territory_type_id': 1},
    {'id': 25, 'initials': 'PB', 'name': 'Paraíba',
     'territory_type_id': 1},
    {'id': 26, 'initials': 'PE', 'name': 'Pernambuco',
     'territory_type_id': 1},
    {'id': 27, 'initials': 'AL', 'name': 'Alagoas',
     'territory_type_id': 1},
    {'id': 28, 'initials': 'SE', 'name': 'Sergipe',
     'territory_type_id': 1},
    {'id': 29, 'initials': 'BA', 'name': 'Bahia', 'territory_type_id': 1},
    {'id': 31, 'initials': 'MG', 'name': 'Minas Gerais',
     'territory_type_id': 1},
    {'id': 32, 'initials': 'ES', 'name': 'Espírito Santo',
     'territory_type_id': 1},
    {'id': 33, 'initials': 'RJ', 'name': 'Rio de Janeiro',
     'territory_type_id': 1},
    {'id': 35, 'initials': 'SP', 'name': 'São Paulo',
     'territory_type_id': 1},
    {'id': 41, 'initials': 'PR', 'name': 'Paraná', 'territory_type_id': 1},
    {'id': 42, 'initials': 'SC', 'name': 'Santa Catarina',
     'territory_type_id': 1},
    {'id': 43, 'initials': 'RS', 'name': 'Rio Grande do Sul',
     'territory_type_id': 1},
    {'id': 50, 'initials': 'MS', 'name': 'Mato Grosso do Sul',
     'territory_type_id': 1},
    {'id': 51, 'initials': 'MT', 'name': 'Mato Grosso',
     'territory_type_id': 1},
    {'id': 52, 'initials': 'GO', 'name': 'Goiás', 'territory_type_id': 1},
    {'id': 53, 'initials': 'DF', 'name': 'Distrito Federal',
     'territory_type_id': 1},
    {'id': 99, 'initials': 'NI', 'name': 'Não informado',
     'territory_type_id': 1},
    {'id': 0, 'initials': 'BR', 'name': 'Brasil', 'territory_type_id': 4},
    {'id': 1003, 'initials': 'RegC', 'name': 'Regional Centro',
     'territory_type_id': 2},
    {'id': 1002, 'initials': 'RegL', 'name': 'Regional Leste',
     'territory_type_id': 2},
    {'id': 1001, 'initials': 'RegN', 'name': 'Regional Norte',
     'territory_type_id': 2},
    {'id': 1004, 'initials': 'RegS', 'name': 'Regional Sul',
     'territory_type_id': 2},
    {'id': 9999, 'initials': 'RegSNI', 'name': 'Regional não informada',
     'territory_type_id': 2},
    {'id': 1, 'initials': 'N', 'name': 'Norte', 'territory_type_id': 3},
    {'id': 2, 'initials': 'NE', 'name': 'Nordeste',
     'territory_type_id': 3},
    {'id': 3, 'initials': 'SE', 'name': 'Sudeste', 'territory_type_id': 3},
    {'id': 5, 'initials': 'CO', 'name': 'Centro-oeste',
     'territory_type_id': 3},
    {'id': 4, 'initials': 'S', 'name': 'Sul', 'territory_type_id': 3},
    {'id': 9, 'initials': 'RNI', 'name': 'Região não informada',
     'territory_type_id': 3},
])


contingency_name_from_id = {
    0: 'Nível basal',
    1: 'Nível 0',
    2: 'Nível 1',
    3: 'Nível 2',
}


def update_data_files(force: bool):
    path_data = os.path.join(PATH, 'data')

    update_params = '-nc' if not force else '-N'
    wget_prefix = (
        ('wget %s ' % update_params) +
        'https://raw.githubusercontent.com/FluVigilanciaBR/data/master/data'
    )

    command = '''cd %(path_data)s; \
    %(wget_prefix)s/br-states.json; \
    %(wget_prefix)s/clean_data_epiweek-weekly-incidence_w_situation.csv && \
    %(wget_prefix)s/contingency_level.csv && \
    %(wget_prefix)s/current_estimated_values.csv && \
    %(wget_prefix)s/historical_estimated_values.csv && \
    %(wget_prefix)s/mem-report.csv && \
    %(wget_prefix)s/mem-typical.csv && \
    %(wget_prefix)s/season_level.csv && \
    %(wget_prefix)s/weekly_alert.csv && \
    %(wget_prefix)s/delay_table.csv''' % {
        'path_data': path_data,
        'wget_prefix': wget_prefix
    }
    print(command.replace('&&', ' && \ \n'))
    os.system(command)
    print('[II] DONE!')


def get_filename_from_path(file_path: str):
    """
    """
    return file_path.split(os.path.sep)[-1].split('.')[0]


def migrate_from_csv_to_psql():
    """

    :return:
    """
    path_data_files = os.path.join(PATH, 'data', '*.csv')

    dfs = {}
    pks = {}

    print('Data files:')
    for file_path in glob.glob(path_data_files):
        filename = get_filename_from_path(file_path)

        print(filename)

        dfs[filename] = pd.read_csv(file_path)

    # ## 1. Setting IDs
    # ### 1.1 Datasets

    df_territory.set_index('id', inplace=True)

    # ## 2. current_estimated_values

    dataset = 'current_estimated_values'

    migration_rules = {
        'UF': 'territory_id',
        'SRAG': 'value',
        'Tipo': 'territory_type',  # Not needed in the table
        'Situation': 'situation_id',
        '50%': 'median',
        '2.5%': 'ci_lower',
        '97.5%': 'ci_upper',
        'L0': 'low_level',
        'L1': 'epidemic_level',
        'L2': 'high_level',
        'L3': 'very_high_level',
        'Run date': 'run_date',
        'dado': 'dataset_id',
        'escala': 'scale_id'
    }

    # rename columns
    dfs[dataset].rename(
        columns=migration_rules, inplace=True
    )

    # apply categories
    dfs[dataset].dataset_id = dfs[dataset].dataset_id.map(dataset_id)
    dfs[dataset].scale_id = dfs[dataset].scale_id.map(scale_id)
    dfs[dataset].situation_id = dfs[dataset].situation_id.map(situation_id)
    regions_indeces = dfs[dataset].territory_id.isin([
        'BR', 'RegN', 'RegL', 'RegC', 'RegS', 'RegNI',
        'N', 'NE', 'SE', 'S', 'CO', 'RNI'
    ])
    dfs[dataset].loc[regions_indeces, 'territory_id'] = dfs[dataset].loc[
        regions_indeces, 'territory_id'
    ].map(region_id)
    dfs[dataset].territory_id = dfs[dataset].territory_id.astype(int)

    # remove unnecessary fields
    dfs[dataset].drop(['territory_type'], axis=1, inplace=True)

    # primary_keys
    pks[dataset] = ['dataset_id', 'scale_id', 'territory_id', 'epiyear',
                    'epiweek']

    dfs[dataset].set_index(pks[dataset], inplace=True)

    # ## 3. historical_estimated_values

    dataset = 'historical_estimated_values'

    migration_rules = {
        'UF': 'territory_id',
        'SRAG': 'value',
        'Tipo': 'territory_type',  # Not needed in the table
        'Situation': 'situation_id',
        '50%': 'median',
        '2.5%': 'ci_lower',
        '97.5%': 'ci_upper',
        'L0': 'low_level',
        'L1': 'epidemic_level',
        'L2': 'high_level',
        'L3': 'very_high_level',
        'Run date': 'run_date',
        'dado': 'dataset_id',  # or origin
        'escala': 'scale_id'
    }

    dfs[dataset].rename(
        columns=migration_rules, inplace=True
    )
    # apply categories
    dfs[dataset].dataset_id = dfs[dataset].dataset_id.map(dataset_id)
    dfs[dataset].scale_id = dfs[dataset].scale_id.map(scale_id)
    dfs[dataset].situation_id = dfs[dataset].situation_id.map(situation_id)
    regions_indeces = dfs[dataset].territory_id.isin([
        'BR', 'RegN', 'RegL', 'RegC', 'RegS', 'RegNI',
        'N', 'NE', 'SE', 'S', 'CO', 'RNI'
    ])
    dfs[dataset].loc[regions_indeces, 'territory_id'] = dfs[dataset].loc[
        regions_indeces, 'territory_id'
    ].map(region_id)
    dfs[dataset].territory_id = dfs[dataset].territory_id.astype(int)

    # remove unnecessary fields
    dfs[dataset].drop(['territory_type'], axis=1, inplace=True)

    # primary_keys
    pks[dataset] = [
        'dataset_id', 'scale_id', 'territory_id',
        'base_epiyear', 'base_epiweek',
        'epiyear', 'epiweek'
    ]

    dfs[dataset].set_index(pks[dataset], inplace=True)
    dfs[dataset].head()

    # ## 4. clean_data_epiweek-weekly-incidence_w_situation

    dataset = 'clean_data_epiweek-weekly-incidence_w_situation'

    migration_rules = {
        '0-4 anos': 'years_0_4',
        '10-19 anos': 'years_10_19',
        '2-4 anos': 'years_2_4',
        '20-29 anos': 'years_20_29',
        '30-39 anos': 'years_30_39',
        '40-49 anos': 'years_40_49',
        '5-9 anos': 'years_5_9',
        '50-59 anos': 'years_50_59',
        '60+ anos': 'years_60_or_more',
        '< 2 anos': 'years_lt_2',
        'DELAYED': 'delayed',
        'FLU_A': 'flu_a',
        'FLU_B': 'flu_b',
        'INCONCLUSIVE': 'inconclusive',
        'Idade desconhecida': 'unknown_age',
        'NEGATIVE': 'negative',
        'NOTTESTED': 'not_tested',
        'OTHERS': 'others',
        'POSITIVE_CASES': 'positive_cases',
        'SRAG': 'value',
        'Situation': 'situation_id',
        'TESTING_IGNORED': 'testing_ignored',
        'Tipo': 'territory_type',  # Not needed in the table
        'UF': 'territory_id',
        'Unidade da Federação': 'state_country_name',  # Not needed
        'VSR': 'vsr',
        'dado': 'dataset_id',
        'escala': 'scale_id',
        'sexo': 'gender'
    }

    dfs[dataset].rename(
        columns=migration_rules, inplace=True
    )
    # apply categories
    dfs[dataset].dataset_id = dfs[dataset].dataset_id.map(dataset_id)
    dfs[dataset].scale_id = dfs[dataset].scale_id.map(scale_id)
    dfs[dataset].situation_id = dfs[dataset].situation_id.map(situation_id)
    regions_indeces = dfs[dataset].territory_id.isin([
        'BR', 'RegN', 'RegL', 'RegC', 'RegS', 'RegNI',
        'N', 'NE', 'SE', 'S', 'CO', 'RNI'
    ])
    dfs[dataset].loc[regions_indeces, 'territory_id'] = dfs[dataset].loc[
        regions_indeces, 'territory_id'
    ].map(region_id)
    dfs[dataset].territory_id = dfs[dataset].territory_id.astype(int)

    # remove unnecessary fields
    dfs[dataset].drop([
        'state_country_name', 'territory_type'
    ], axis=1, inplace=True)

    # primary_keys
    pks[dataset] = ['dataset_id', 'scale_id', 'territory_id', 'epiyear',
                    'epiweek']

    dfs[dataset].set_index(pks[dataset], inplace=True)
    dfs[dataset].head()

    # ## 5. mem-report

    dataset = 'mem-report'

    migration_rules = {
        'UF': 'territory_id',
        'População': 'population',
        'Média geométrica do pico de infecção das temporadas regulares':
            'geom_average_peak',
        'região de baixa atividade típica': 'low_activity_region',
        'limiar pré-epidêmico': 'pre_epidemic_threshold',
        'intensidade alta': 'high_threshold',
        'intensidade muito alta': 'very_high_threshold',
        'SE típica do início do surto': 'epi_start',
        'SE típica do início do surto - IC inferior (2,5%)':
            'epi_start_ci_lower',
        'SE típica do início do surto - IC superior (97,5%)':
            'epi_start_ci_upper',
        'duração típica do surto': 'epi_duration',
        'duração típica do surto - IC inferior (2,5%)':
            'epi_duration_ci_lower',
        'duração típica do surto - IC superior (97,5%)':
            'epi_duration_ci_upper',
        'temporadas utilizadas para os corredores endêmicos':
            'regular_seasons',
        'ano': 'year',
        'Unidade da Federação': 'state_country_name',  # Not needed ...
        'Tipo': 'territory_type',  # Not needed in the table
        'dado': 'dataset_id',
        'escala': 'scale_id'
    }

    dfs[dataset].rename(
        columns=migration_rules, inplace=True
    )
    # apply categories
    dfs[dataset].dataset_id = dfs[dataset].dataset_id.map(dataset_id)
    dfs[dataset].scale_id = dfs[dataset].scale_id.map(scale_id)
    regions_indeces = dfs[dataset].territory_id.isin([
        'BR', 'RegN', 'RegL', 'RegC', 'RegS', 'RegNI',
        'N', 'NE', 'SE', 'S', 'CO', 'RNI'
    ])
    dfs[dataset].loc[regions_indeces, 'territory_id'] = dfs[dataset].loc[
        regions_indeces, 'territory_id'
    ].map(region_id)
    dfs[dataset].territory_id = dfs[dataset].territory_id.astype(int)

    # remove unnecessary fields
    dfs[dataset].drop([
        'state_country_name', 'territory_type'
    ], axis=1, inplace=True)

    # primary_keys
    pks[dataset] = ['dataset_id', 'scale_id', 'territory_id', 'year']

    dfs[dataset].set_index(pks[dataset], inplace=True)

    # ## 6. mem-typical

    dataset = 'mem-typical'

    migration_rules = {
        'UF': 'territory_id',
        'População': 'population',
        'corredor baixo': 'low',
        'corredor mediano': 'median',
        'corredor alto': 'high',
        'ano': 'year',
        'Unidade da Federação': 'state_country_name',  # Not needed ...
        'Tipo': 'territory_type',  # Not needed in the table
        'dado': 'dataset_id',
        'escala': 'scale_id'
    }

    dfs[dataset].rename(
        columns=migration_rules, inplace=True
    )
    # apply categories
    dfs[dataset].dataset_id = dfs[dataset].dataset_id.map(dataset_id)
    dfs[dataset].scale_id = dfs[dataset].scale_id.map(scale_id)
    regions_indeces = dfs[dataset].territory_id.isin([
        'BR', 'RegN', 'RegL', 'RegC', 'RegS', 'RegNI',
        'N', 'NE', 'SE', 'S', 'CO', 'RNI'
    ])
    dfs[dataset].loc[regions_indeces, 'territory_id'] = dfs[dataset].loc[
        regions_indeces, 'territory_id'
    ].map(region_id)
    dfs[dataset].territory_id = dfs[dataset].territory_id.astype(int)

    # remove unnecessary fields
    dfs[dataset].drop([
        'state_country_name', 'territory_type'
    ], axis=1, inplace=True)

    # primary_keys
    pks[dataset] = ['dataset_id', 'scale_id', 'territory_id', 'year',
                    'epiweek']

    dfs[dataset].set_index(pks[dataset], inplace=True)

    # ## 7. delay_table

    dataset = 'delay_table'

    migration_rules = {
        'UF': 'territory_id',
        'Notific2Digita_DelayDays': 'notification2digitalization',
        'SinPri2Digita_DelayDays': 'symptoms2digitalization',
        'SinPri2Antivir_DelayDays': 'symptoms2antiviral',
        'SinPri2Notific_DelayDays': 'symptoms2notification',
        'SinPri2Coleta_DelayDays': 'symptoms2sample',
        'Notific2Encerra_DelayDays': 'notification2closure',
        'Coleta2IFI_DelayDays': 'sample2ifi',
        'Coleta2PCR_DelayDays': 'sample2pcr',
        'Regional': 'regional',
        'Regiao': 'region',
        'dado': 'dataset_id'
    }

    dfs[dataset].rename(columns=migration_rules, inplace=True)

    # apply categories
    dfs[dataset].dataset_id = dfs[dataset].dataset_id.map(dataset_id)
    dfs[dataset].territory_id = dfs[dataset].territory_id.astype(int)
    dfs[dataset].regional = dfs[dataset].regional.map(region_id).astype(int)
    dfs[dataset].region = dfs[dataset].region.map(region_id).astype(int)

    # remove unnecessary fields
    dfs[dataset].drop([
        'Notific2Digita_DelayWeeks',
        'SinPri2Digita_DelayWeeks',
        'SinPri2Antivir_DelayWeeks',
        'SinPri2Notific_DelayWeeks',
        'SinPri2Coleta_DelayWeeks',
        'Notific2Encerra_DelayWeeks',
        'Coleta2IFI_DelayWeeks',
        'Coleta2PCR_DelayWeeks',
    ], axis=1, inplace=True)

    # add index
    dfs[dataset]['id'] = dfs[dataset].index

    # primary keys
    pks[dataset] = ['id', 'territory_id', 'epiyear', 'epiweek']
    dfs[dataset].set_index(pks[dataset], inplace=True)

    # ## SQL Migration

    exception_type_field = {
        'run_date': 'DATE'
    }

    # creating dataset dataframe
    df_dataset = pd.DataFrame({
        'id': list(dataset_id.values()),
        'name': list(dataset_id.keys())
    }).set_index('id')

    dfs['dataset'] = df_dataset

    # creating situation dataframe
    df_situation = pd.DataFrame({
        'id': list(situation_id.values()),
        'name': list(situation_id.keys())
    }).set_index('id')

    dfs['situation'] = df_situation

    # creating scale dataframe
    df_scale = pd.DataFrame({
        'id': list(scale_id.values()),
        'name': list(scale_id.keys())
    }).set_index('id')

    dfs['scale'] = df_scale

    # creating territory_type dataframe
    df_territory_type = pd.DataFrame({
        'id': list(territory_type_id.values()),
        'name': list(territory_type_id.keys())
    }).set_index('id')

    dfs['territory_type'] = df_territory_type

    dfs['territory'] = df_territory

    dsn = 'postgresql://%(USER)s:%(PASSWORD)s@%(HOST)s/%(NAME)s'
    engine = sqla.create_engine(dsn % DATABASE)

    for k, df in dfs.items():
        k_new = k.replace('-', '_')

        print('Migrating %s ...' % k_new)
        df.to_sql(
            k_new, engine, if_exists='replace',
            chunksize=2048
        )


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description="Update datafiles and DB.\n",
                                     formatter_class=RawDescriptionHelpFormatter)
    parser.add_argument('-F', '--Force', help='Force download', action='store_true')
    parser.add_argument('-d', '--database', help='Update database', action='store_true')

    args = parser.parse_args()
    update_data_files(force=args.Force)
    if args.database:
        migrate_from_csv_to_psql()



from . import settings

import os
import sys


def startup():
    try:
        import gunicorn
    except:
        raise Exception('[EE] GUNICORN NOT FOUND.')

    path_root = os.path.dirname(os.path.abspath(__file__))
    path_file = os.path.join(path_root, 'runwsgi.sh')

    for setting_variable in dir(settings):
        if setting_variable.startswith("__"):
            continue

        v = settings.__dict__[setting_variable]

        # apply just for string variables
        if isinstance(v, str):
            os.environ[setting_variable] = v

    os.environ['PYTHONPATH'] = os.path.dirname(sys.executable)
    os.environ['PATH'] += ':' + os.environ['PYTHONPATH']
    os.system('python --version')
    os.system('/bin/bash %s' % path_file)


if __name__ == '__main__':
    startup()



from datetime import timedelta
from flask import make_response, request, current_app
from functools import update_wrapper
# local
from .episem import episem, extractweekday as extract_weekday

import datetime
import os


def calc_last_epiweek(year):
    # Extract last Brazilian epiweek from given year

    day = datetime.datetime(int(year), 12, 31)  # Ultimo dia do ano
    day_week = extract_weekday(day)  # dia semana do ultimo dia

    if day_week < 3:
        day = day - datetime.timedelta(days=(day_week+1))
    else:
        day = day + datetime.timedelta(days=(6-day_week))

    return int(episem(day, out='W'))


def cross_domain(
    origin=None, methods=None, headers=None,
    max_age=21600, attach_to_all=True,
    automatic_options=True
):
    """

    :see: http://flask.pocoo.org/snippets/56/
    :param origin:
    :param methods:
    :param headers:
    :param max_age:
    :param attach_to_all:
    :param automatic_options:
    :return:
    """
    basestring = str

    if methods is not None:
        methods = ', '.join(sorted(x.upper() for x in methods))
    if headers is not None and not isinstance(headers, basestring):
        headers = ', '.join(x.upper() for x in headers)
    if not isinstance(origin, basestring):
        origin = ', '.join(origin)
    if isinstance(max_age, timedelta):
        max_age = max_age.total_seconds()

    def get_methods():
        if methods is not None:
            return methods

        options_resp = current_app.make_default_options_response()
        return options_resp.headers['allow']

    def decorator(f):
        def wrapped_function(*args, **kwargs):
            if automatic_options and request.method == 'OPTIONS':
                resp = current_app.make_default_options_response()
            else:
                resp = make_response(f(*args, **kwargs))
            if not attach_to_all and request.method != 'OPTIONS':
                return resp

            h = resp.headers

            h['Access-Control-Allow-Origin'] = origin
            h['Access-Control-Allow-Methods'] = get_methods()
            h['Access-Control-Max-Age'] = str(max_age)
            if headers is not None:
                h['Access-Control-Allow-Headers'] = headers
            return resp

        f.provide_automatic_options = False
        return update_wrapper(wrapped_function, f)
    return decorator


def recursive_dir_name(path: str, steps_back: int):
    """

    :param path:
    :param steps_back:
    :return:
    """
    for _ in range(steps_back):
        path = os.path.dirname(path)
    return path




import numpy as np
import datetime

__author__ = 'Marcelo Ferreira da Costa Gomes'
'''
Return Brazilian epidemiological week from passed date
'''


def extractweekday(x=datetime.datetime):
    # Extract weekday as [Sun-Sat] |-> [0-6]
    w = x.isoweekday() % 7  # isoweekday() returns weekday with [Mon-Sun] as [1-7]
    return w


def firstepiday(year=int):
    day = datetime.datetime.strptime('%s-01-01' % year, '%Y-%m-%d')


    day_week = extractweekday(day)

    # Whe want day1 to correspond to the first day of the first epiweek. That is, we need
    # the Sunday corresponding to epiweek=%Y01
    # If first day of the year is between Sunday and Wednesday, epiweek 01 includes it
    # Otherwise, it is still the last epiweek of the previous year
    if day_week < 4:
        day = day - datetime.timedelta(days=day_week)
    else:
        day = day + datetime.timedelta(days=(7-day_week))

    return day


def lastepiday(year=int):
    day = datetime.datetime.strptime('%s-12-31' % year, '%Y-%m-%d')


    day_week = extractweekday(day)

    # Whe want day to correspond to the last day of the last epiweek. That is, we need
    # the corresponding Saturday
    # If the last day of the year is between Sunday and Tuesday, epiweek 01 of the next year includes it.
    # Otherwise, it is still the last epiweek of the current year
    if day_week < 3:
        day = day - datetime.timedelta(days=(day_week+1))
    else:
        day = day + datetime.timedelta(days=(6-day_week))

    return day


def episem(x, sep='W', out='YW'):

    """
    Return Brazilian corresponding epidemiological week from x.

    :param x: Input date. Can be a string in the format %Y-%m-%d or datetime.datetime
    :param sep: Year and week separator.
    :param out: Output format. 'YW' returns sep.join(epiyear,epiweek).
     'Y' returns epiyear only. 'W' returns epiweek only.
    :return: str
    """

    def out_format(year, week, out):
        if out == 'YW':
            return('%sW%02d' % (year,week))
        if out == 'Y':
            return('%s' % (year))
        if out == 'W':
            return ('%02d' % week)

    if (type(x) != datetime.datetime):
        if (str(x)=='' or (type(x)!=str and np.isnan(x))):
            return(None)
        x = datetime.datetime.strptime(x, '%Y-%m-%d')

    epiyear = x.year
    epiend = lastepiday(epiyear)

    if x > epiend:
        epiyear += 1
        return(out_format(epiyear, 1, out))

    epistart = firstepiday(epiyear)

    # If current date is before its year first epiweek, then our base year is the previous one
    if x < epistart:
        epiyear -= 1
        epistart = firstepiday(epiyear)

    epiweek = int(((x - epistart)/7).days) + 1

    return(out_format(epiyear, epiweek, out))



from unidecode import unidecode
# local
from ..settings import DATABASE

import pandas as pd
import sqlalchemy as sqla


# @deprecated
def prepare_keys_name(df):
    """
    Standardises data frame keys

    :param df:
    :type df: pd.DataFrame
    :return: pd.DataFrame
    """
    for k in df.keys():
        df.rename(columns={
            k: unidecode(
                k.replace(' ', '_').replace('-', '_').lower()
            ).encode('ascii').decode('utf8')
        }, inplace=True)
    return df


class FluDB:
    conn = None

    def __init__(self):
        dsn = 'postgresql://%(USER)s:%(PASSWORD)s@%(HOST)s/%(NAME)s'
        self.conn = sqla.create_engine(dsn % DATABASE)

    def get_territory_id_from_name(self, state_name: str) -> int:
        """

        :param state_name:
        :return:
        """

        state_name = state_name.upper()
        with self.conn.connect() as conn:
            sql = '''
            SELECT id FROM territory 
            WHERE UPPER(name)='%s'
            ''' % state_name

            result = conn.execute(sql).fetchone()

            if not result:
                raise Exception('State not found.')

            return result[0]

    def get_territory_from_name(self, state_name: str) -> int:
        """

        :param state_name:
        :return:
        """

        state_name = state_name.upper()
        with self.conn.connect() as conn:
            sql = '''
            SELECT * FROM territory 
            WHERE UPPER(name)='%s'
            ''' % state_name

            result = conn.execute(sql).fetchone()

            if not result:
                raise Exception('Territory not found.')

            return result

    def get_season_situation(self, df):
        """

        :param df:
        :return:
        """
        def _fn(se):
            return df[
                (df.territory_id == se.territory_id) &
                (df.epiyear == se.epiyear)
            ].situation_id.unique()[0]
        return _fn

    def get_season_level(self, se):
        """
        Generate season level code based on counts over weekly activity

        """
        if se.high_level + se.very_high_level > 4:
            return 4  # 'red'
        elif se.high_level + se.very_high_level >= 1:
            return 3  # 'orange'
        elif se.epidemic_level >= 1:
            return 2  # 'yellow'
        # else
        return 1  # 'green' or 'white'

    def group_data_by_season(self, df, df_age_dist=None, season=None):
        """

        :param df:
        :param df_age_dist:
        :param season:
        :return:
        """
        level_dict = {
            'low_level': 'Baixa', 'epidemic_level': 'Epidêmica',
            'high_level': 'Alta', 'very_high_level': 'Muito alta'
        }
        season_basic_cols = [
            'territory_id', 'territory_name', 'epiyear', 'value'
        ]
        season_cols = season_basic_cols + [
            'territory_type_name', 'situation_id', 'situation_name', 'level'
        ]

        df['level'] = df[list(level_dict.keys())].idxmax(axis=1)

        df_tmp = df[season_cols].copy()

        situation = list(
            df_tmp[df_tmp.epiyear == season].situation_id.unique()
        )
        l_incomplete = [1, 2, 4]
        if set(l_incomplete).intersection(situation):
            # incomplete
            df_tmp.loc[df_tmp.epiyear == season, 'situation_id'] = 4
        else:
            # stable
            df_tmp.loc[df_tmp.epiyear == season, 'situation_id'] = 3

        if df_age_dist is not None:
            tgt_cols = [
                'territory_id', 'territory_name', 'epiyear', 'gender',
                'value', 'years_lt_2', 'years_2_4', 'years_0_4', 'years_5_9',
                'years_10_19', 'years_20_29', 'years_30_39', 'years_40_49',
                'years_50_59', 'years_60_or_more'
            ]

            df_by_season = df_age_dist[tgt_cols].groupby([
                'territory_id', 'territory_name', 'epiyear', 'gender'
            ], as_index=False).sum()
        else:
            df_by_season = df_tmp[season_basic_cols].groupby(
                ['territory_id', 'territory_name', 'epiyear'],
                as_index=False
            ).sum()

        situations_id = {
            1: 'unknown',
            2: 'estimated',
            3: 'stable',
            4: 'incomplete'
        }

        df_by_season['situation_id'] = df_by_season.apply(
            self.get_season_situation(df_tmp), axis=1
        )

        df_by_season['situation_name'] = df_by_season['situation_id'].map(
            situations_id
        )

        df_by_season_level = pd.crosstab([
            df_tmp.territory_id, df_tmp.territory_name, df_tmp.epiyear
        ], df_tmp.level).reset_index()

        df_by_season_level.columns.name = None

        for lv in (
            'low_level', 'epidemic_level', 'high_level', 'very_high_level'
        ):
            if not lv in df_by_season_level.keys():
                df_by_season_level[lv] = 0

        df_by_season['level'] = df_by_season_level[
            list(level_dict.keys())
        ].apply(
            self.get_season_level, axis=1
        )

        df_by_season['epiweek'] = 0

        return df_by_season

    def report_incidence(self, x, situation, low=None, high=None):
        """
        original name: report_inc

        :param x:
        :param situation:
        :param low:
        :param high:
        :return:
        """
        if situation == 3:
            y = '%.2f' % x
        elif situation == 2:
            y = '%.2f [%.2f - %.2f]' % (x, low, high)
        else:
            y = '*%.2f' % x
        return y

    def read_data(
        self, table_name: str, dataset_id: int, scale_id: int,
        territory_id: int=None, year: int=None, week: int=None,
        base_year: int=None, base_week: int=None,
        historical_week: int=None, return_sql=False,
        extra_fields: list=None, selected_fields: list=None,
        excluded_fields: list=[], **kwargs
    ):
        """

        :param table_name:
        :param dataset_id:
        :param scale_id:
        :param territory_id:
        :param year:
        :param week:
        :param base_year:
        :param base_week:
        :param historical_week:
        :param return_sql:
        :param extra_fields:
        :param selected_fields:
        :param excluded_fields:
        :param kwargs:
        :return:

        """
        if selected_fields is None:
            # get all fields from the table
            sql = '''
            SELECT column_name
            FROM information_schema.columns
            WHERE table_name   = '%s'
            ORDER BY ordinal_position
            ''' % table_name

            with self.conn.connect() as conn:
                selected_fields = conn.execute(sql).fetchall()

            selected_fields = [
                f[0] for f in selected_fields
                if f[0] not in ['dataset_id', 'scale_id'] + excluded_fields
            ]

        if extra_fields is not None:
            selected_fields += extra_fields

        sql_param = {
            'table_name': table_name,
            'dataset_id': dataset_id,
            'scale_id': scale_id,
            'fields': ','.join(selected_fields)
        }

        sql = '''
        SELECT %(fields)s, territory.name AS territory_name
        FROM %(table_name)s 
          INNER JOIN territory
            ON (%(table_name)s.territory_id = territory.id)
        WHERE dataset_id=%(dataset_id)s 
          AND scale_id=%(scale_id)s
        '''

        if territory_id is not None:
            sql += " AND territory_id=%(territory_id)s"
            sql_param['territory_id'] = territory_id

        if year is not None:
            sql += " AND epiyear=%(year)s"
            sql_param['year'] = year

        if base_year is not None:
            sql += " AND base_epiyear=%(base_year)s"
            sql_param['base_year'] = base_year

        if week is not None and week > 0:
            sql += " AND epiweek=%(week)s"
            sql_param['week'] = week
        elif base_week is not None:
            sql += " AND base_epiweek=%(base_week)s"
            sql_param['base_week'] = base_week
        elif historical_week is not None:
            sql += " AND epiweek<=%(historical_week)s"
            sql_param['historical_week'] = historical_week

        if return_sql:
            return sql % sql_param

        with self.conn.connect() as conn:
            return pd.read_sql(sql % sql_param, conn)

    def get_data(
        self, dataset_id: int, scale_id: int, year: int,
        territory_id: int=None, week: int=None,
        show_historical_weeks: bool=False,
        territory_type_id: int=None
    ):
        """

        :param dataset_id:
        :param scale_id:
        :param year:
        :param territory_id:
        :param week: 0 week == all weeks
        :param show_historical_weeks:
        :param territory_type_id:
        :return:
        """
        sql = '''
        SELECT
          mem_typical.dataset_id AS dataset_id,
          mem_typical.scale_id AS scale_id,
          mem_typical.territory_id AS territory_id,
          incidence.epiyear AS epiyear,
          mem_typical.epiweek AS epiweek,
          incidence.value, 
          incidence.low_level as low_level,
          incidence.epidemic_level as epidemic_level,
          incidence.high_level as high_level,
          incidence.very_high_level as very_high_level,
          incidence.situation_id AS situation_id,
          incidence.run_date,
          %(estimates_columns_selection)s
          mem_typical.population, 
          mem_typical.low AS typical_low, 
          mem_typical.median-mem_typical.low AS typical_median, 
          mem_typical.high-mem_typical.median AS typical_high,
          mem_report.geom_average_peak, 
          mem_report.low_activity_region, 
          mem_report.pre_epidemic_threshold as pre_epidemic_threshold, 
          mem_report.high_threshold as high_threshold, 
          mem_report.very_high_threshold as very_high_threshold, 
          mem_report.epi_start, 
          mem_report.epi_start_ci_lower,
          mem_report.epi_start_ci_upper, 
          mem_report.epi_duration, 
          mem_report.epi_duration_ci_lower, 
          mem_report.epi_duration_ci_upper,
          mem_report.regular_seasons,
          historical.base_epiyear, 
          historical.base_epiweek,
          territory.name AS territory_name,
          territory_type.name AS territory_type_name,
          situation.name AS situation_name,
          contingency_level.contingency AS contingency,
          contingency_level.contingency_max AS contingency_max,
          weekly_alert.alert AS alert,
          season_level.season_level AS season_level
        FROM
          (
            SELECT
            epiyear,
            epiweek,
            dataset_id,
            scale_id,
            territory_id,
            situation_id,
            "value",
            low_level,
            epidemic_level,
            high_level,
            very_high_level,
            run_date 
            %(incidence_table_select)s
            FROM current_estimated_values
            WHERE dataset_id=%(dataset_id)s 
              AND scale_id=%(scale_id)s 
              AND epiyear=%(epiyear)s 
              AND epiweek %(incidence_week_operator)s %(epiweek)s
              %(territory_id_condition)s
              %(situation_id_condition)s
          ) AS incidence 
          INNER JOIN situation
            ON (incidence.situation_id=situation.id)
          FULL OUTER JOIN (
            SELECT * FROM mem_typical
            WHERE dataset_id=%(dataset_id)s 
              AND scale_id=%(scale_id)s
              %(territory_id_condition)s 
            ) AS mem_typical
            ON (
              incidence.dataset_id=mem_typical.dataset_id
              AND incidence.scale_id=mem_typical.scale_id
              AND incidence.territory_id=mem_typical.territory_id
              AND incidence.epiweek=mem_typical.epiweek
            )
          LEFT JOIN (
            SELECT * FROM weekly_alert
            WHERE dataset_id=%(dataset_id)s
              AND epiyear=%(epiyear)s
              %(territory_id_condition)s
            ) AS weekly_alert
            ON (
              mem_typical.dataset_id=weekly_alert.dataset_id
              AND mem_typical.territory_id=weekly_alert.territory_id
              AND mem_typical.epiweek=weekly_alert.epiweek            
            )
          INNER JOIN territory
            ON (mem_typical.territory_id=territory.id)
          INNER JOIN territory_type
            ON (territory.territory_type_id=territory_type.id)
          %(historical_table)s
          FULL OUTER JOIN (
            SELECT * FROM mem_report
            WHERE dataset_id=%(dataset_id)s 
              AND scale_id=%(scale_id)s
              %(territory_id_condition)s
            ) AS mem_report
            ON (
              mem_typical.dataset_id=mem_report.dataset_id
              AND mem_typical.scale_id=mem_report.scale_id
              AND mem_typical.territory_id=mem_report.territory_id
              AND mem_typical.year=mem_report.year
            )
          FULL OUTER JOIN (
            SELECT * FROM contingency_level
            WHERE epiyear=%(epiyear)s
            %(territory_id_condition)s
            ) AS contingency_level
            ON (
            mem_report.territory_id=contingency_level.territory_id
            )
          FULL OUTER JOIN (
            SELECT * FROM season_level
            WHERE epiyear=%(epiyear)s
            %(territory_id_condition)s
            ) AS season_level
            ON (
            mem_report.territory_id=season_level.territory_id
            AND mem_typical.dataset_id=season_level.dataset_id

            )
         WHERE 1=1
           %(where_extras)s
        ORDER BY epiyear, epiweek
        '''

        sql_param = {
            'dataset_id': dataset_id,
            'scale_id': scale_id,
            'territory_id': territory_id,
            'epiweek': week,
            'epiyear': year,
            'epiweekstop': 54,
            'base_epiweek_condition': '',
            'estimates_columns_selection': '''
            incidence.mean  AS "mean",
            incidence.median AS estimated_cases, 
            incidence.ci_lower AS ci_lower, 
            incidence.ci_upper AS ci_upper, 
            ''',
            'where_extras': '',
            'historical_table': '''
            FULL OUTER JOIN (
              SELECT * 
              FROM historical_estimated_values LIMIT 0
            ) AS historical ON (1=1)
            ''',
            'incidence_week_operator': '=',
            'territory_id_condition': '',
            'situation_id_condition': '',
            'incidence_table_select': ''' 
            ,mean,
            median,
            ci_lower,
            ci_upper
            '''

        }

        if territory_id is not None:
            sql_param['territory_id_condition'] += (
                ' AND territory_id=%s ' % territory_id
            )

        if week is None or week == 0:
            sql_param['epiweek'] = 54
            sql_param['incidence_week_operator'] = '<='
            sql_param['base_epiweek_condition'] = '''
            AND base_epiweek = (
                SELECT MAX(base_epiweek)
                FROM historical_estimated_values
                WHERE base_epiyear = %(epiyear)s\
                AND dataset_id = %(dataset_id)s
                AND scale_id = %(scale_id)s
                %(territory_id_condition)s )
            ''' % sql_param
            with self.conn.connect() as conn:
                sql_param['epiweekstop'] = conn.execute('''
                SELECT MAX(epiweek) FROM current_estimated_values
                WHERE epiyear = %(epiyear)s
                  AND "value" IS NOT NULL
                ''' % sql_param).fetchone()[0] - 4
        else:
            sql_param['epiweekstop'] = week - 4
            sql_param['base_epiweek_condition'] = '''
            AND base_epiweek = (
                SELECT MAX(LEAST(base_epiweek, %(epiweek)s))
                FROM historical_estimated_values
                WHERE base_epiyear=%(epiyear)s
                    AND dataset_id = %(dataset_id)s
                    AND scale_id = %(scale_id)s
                    AND base_epiweek <= %(epiweek)s
                    %(territory_id_condition)s )
            AND epiyear =%(epiyear)s
            ''' % sql_param

        # force week filter (week 0 == all weeks)
        if show_historical_weeks:
            with self.conn.connect() as conn:
                epiyearmax = conn.execute('SELECT MAX(epiyear) FROM current_estimated_values').fetchone()[0]
            if year < epiyearmax:
                sql_param['situation_id_condition'] = ' AND situation_id = 3'
            else:
                sql_param['situation_id_condition'] = ' AND epiweek <= %(epiweekstop)s' % sql_param
            sql_param['estimates_columns_selection'] = '''
            historical.mean  AS mean,
            historical.median AS estimated_cases, 
            historical.ci_lower AS ci_lower, 
            historical.ci_upper AS ci_upper, 
            '''
            sql_param['incidence_table_select'] = ''
            sql_param['historical_table'] = '''
          LEFT JOIN (
            SELECT territory_id,
                epiweek,
                mean,
                median,
                ci_lower,
                ci_upper,
                low_level,
                epidemic_level,
                high_level,
                very_high_level,
                base_epiyear,
                base_epiweek,
                base_epiyearweek
            FROM historical_estimated_values
            WHERE dataset_id=%(dataset_id)s 
             AND scale_id=%(scale_id)s 
             AND territory_id=%(territory_id)s 
             AND base_epiyear=%(epiyear)s
             AND situation_id = 2
             %(base_epiweek_condition)s
          ) AS historical
            ON (
              mem_typical.epiweek=historical.epiweek
            )
            ''' % sql_param
            sql_param['incidence_week_operator'] = '<='
        else:
            sql_param['where_extras'] += ' AND incidence.epiweek %s %s' % (
                sql_param['incidence_week_operator'], sql_param['epiweek']
            )

        if territory_type_id is not None and territory_type_id > 0:
            sql_param['where_extras'] += (
                ' AND territory.territory_type_id=%s' % territory_type_id
            )

        sql = sql % sql_param

        with self.conn.connect() as conn:
            return pd.read_sql(sql, conn)

    def get_data_age_sex(
        self, dataset_id: int, scale_id: int, year: int,
        territory_id: int=0, week: int=0
    ):
        """

        :param dataset_id:
        :param scale_id:
        :param year:
        :param territory_id:
        :param week:
        :return:

        """
        season = year  # alias

        if scale_id == 1:
            age_cols = ['years_0_4']
        else:
            age_cols = ['years_lt_2', 'years_2_4']

        age_cols.extend([
            'years_5_9', 'years_10_19', 'years_20_29',
            'years_30_39', 'years_40_49', 'years_50_59', 'years_60_or_more'
        ])

        # data
        df_age_dist = self.read_data(
            'clean_data_epiweek_weekly_incidence_w_situation',
            dataset_id=dataset_id, scale_id=scale_id, year=season,
            territory_id=territory_id, low_memory=False, excluded_fields=[
                'ADNO', 'PARA1', 'PARA2', 'PARA3'
            ]
        )

        if week is not None and week > 0:
            df_age_dist = df_age_dist[df_age_dist.epiweek == week]
            df = df_age_dist
        else:
            df = self.get_data(
                dataset_id=dataset_id, scale_id=scale_id,
                year=year, territory_id=territory_id
            )
            df = self.group_data_by_season(
                df=df, df_age_dist=df_age_dist, season=season
            )

        df = df[age_cols + ['gender']].set_index('gender').transpose()
        df.rename(columns={'F': 'Mulheres', 'M': 'Homens'}, inplace=True)
        if 'I' in df.columns:
            df.rename(columns={'I': 'Sexo ignorado'}, inplace=True)
            df = df[['Mulheres', 'Homens', 'Sexo ignorado', 'Total']]

        return df

    def get_etiological_data(
        self, dataset_id: int, scale_id: int, year: int,
        week: int=None, territory_id: int=None
    ) -> pd.DataFrame:
        """
        Generate timeseries for each ethiological agent and other relevant lab
        data

        :param dataset_id: SRAG (1) or SRAGFLU (2) or OBITOFLU (3)
        :param scale_id: Incidence (1) or cases (2)
        :param year: epidemiological year
        :param week: epidemiological week or all (None or 0)
        :param territory_id:
        """

        sql_param = {
            'epiweek': week,
            'epiyear': year,
            'territory_id': territory_id,
            'dataset_id': dataset_id,
            'scale_id': scale_id
        }

        if week == None or week == 0:
            sql_param['epiweek'] = 53
        if territory_id == None:
            sql_param['territory_id'] = 0

        sql = '''
        SELECT
          notification.epiweek AS epiweek,
          notification.positive_cases AS "Testes positivos",
          notification.flu_a AS "Influenza A",
          notification.flu_b AS "Influenza B",
          notification.vsr AS "VSR",
          notification."ADNO" AS "Adenovirus",
          notification."PARA1" AS "Parainfluenza 1",
          notification."PARA2" AS "Parainfluenza 2",
          notification."PARA3" AS "Parainfluenza 3",
          notification.negative AS "Testes negativos",
          notification.not_tested AS "Casos sem teste laboratorial",
          notification.delayed AS "Casos aguardando resultado",
          notification.testing_ignored AS "Casos sem informação laboratorial",
          territory.name AS territory_name
        FROM
          (SELECT 
            epiweek,
            epiyear,
            dataset_id,
            scale_id,
            gender,
            positive_cases,
            flu_a,
            flu_b,
            vsr,
            "ADNO",
            "PARA1",
            "PARA2",
            "PARA3",
            negative,
            not_tested,
            delayed,
            testing_ignored,
            territory_id
          FROM
            clean_data_epiweek_weekly_incidence_w_situation
          WHERE
            epiyear=%(epiyear)s
            AND dataset_id=%(dataset_id)s
            AND territory_id=%(territory_id)s
            AND scale_id=%(scale_id)s
            AND gender='Total'
            AND epiweek <= %(epiweek)s
          ) as notification
        LEFT JOIN territory
            ON (notification.territory_id=territory.id)
        WHERE 1=1
        ORDER BY epiweek
        ''' % sql_param

        with self.conn.connect() as conn:
            return pd.read_sql(sql, conn)

    def get_opportunities(
        self, dataset_id: int, scale_id: int, year: int,
        territory_type_id: int, week: int=None,
        territory_id: int=None
    ) -> pd.DataFrame:
        """
        Grab data for opportunity boxplots

        :param dataset_id:
        :param scale_id:
        :param year: selected year
        :param week: selected week. 0 or Nonoe for all
        :param territory_id: territory. 0 or None for whole country
        :param territory_type_id: territory type. None or 4 for whole country
        :return pd.DataFrame:

        """
        sql_param = {
            'dataset_id': dataset_id,
            'scale_id': scale_id,
            'epiweek': week,
            'epiyear': year,
            'territory_id': territory_id,
            'territory_select': '',
            'territory_name': '',
            'territory_join': ''
        }

        territory_type2column = {
            1: 'territory_id',
            2: 'regional',
            3: 'region'
        }

        if territory_id not in [0, None]:
            sql_param['territory_name'] = ', territory.name AS territory_name'
            sql_param['territory_column'] = territory_type2column[
                territory_type_id
            ]
            sql_param[
                'territory_select'
            ] = '''AND %(territory_column)s = %(territory_id)s''' % sql_param
            sql_param['territory_join'] = '''LEFT JOIN territory
                ON (delay.%(territory_column)s=territory.id)
            WHERE 1=1
            ''' % sql_param

        if week is None or week == 0:
            sql_param['epiweek'] = 53

        sql = '''
        SELECT
        delay.symptoms2notification as "Primeiros sintomas à notificação",
        delay.symptoms2digitalization AS "Primeiros sintomas à digitalização",
        delay.notification2digitalization AS "Notificação à digitalização",
        delay.symptoms2antiviral AS "Primeiros sintomas ao tratamento",
        delay.symptoms2sample AS "Primeiros sintomas à coleta",
        delay.sample2ifi AS "Coleta a resultado de IFI",
        delay.sample2PCR AS "Coleta a resultado de PCR",
        delay.notification2closure AS "Notificação ao encerramento"        
        %(territory_name)s
        FROM
            (SELECT * 
            FROM
                delay_table
            WHERE
                epiyear=%(epiyear)s
                AND dataset_id=%(dataset_id)s
                AND epiweek <= %(epiweek)s
                %(territory_select)s
            ) as delay
            %(territory_join)s
        ''' % sql_param

        with self.conn.connect() as conn:
            df = pd.read_sql(sql, conn)

        if territory_id in [0, None]:
            df['territory_name'] = 'Brasil'
        return df



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, Command
from pip.req import parse_requirements
from pip.download import PipSession

import os

PATH_ROOT = os.path.dirname(os.path.abspath(__file__))


def get_version():
    """Obtain the version number"""
    import importlib
    mod = importlib.machinery.SourceFileLoader(
        'version', os.path.join('fludashboard', '__init__.py')
    ).load_module()
    return mod.__version__


class MakeDoc(Command):
    """
    MakeDoc will prepare Python and JavaScript document from code.

    """
    description = "Prepares documentation"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        """

        :return:
        """
        import sh

        env = os.environ.copy()

        # update RST files from python files
        sh.cd(os.path.join(PATH_ROOT, 'docs'))
        out = sh.sphinx_apidoc('-o', '.', '../fludashboard', '--force')
        print(out)

        # update RST files from python javascript files
        path_template = list(
            sh.grep(sh.locate('jsdoc-sphinx'), 'template$')
        )[0].replace('\n', '')

        out = sh.jsdoc(
            '-t', path_template, '-d', 'jsdoc/', '../fludashboard/static/js/'
        )
        print(out)


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

install_reqs = parse_requirements('requirements.txt', session=PipSession())

requirements = [str(ir.req) for ir in install_reqs]

test_requirements = [
    # TODO: put package test requirements here
]

setup(
    cmdclass={'doc': MakeDoc},
    name='fludashboard',
    version=get_version(),
    description="Flu Dashboard",
    long_description=readme + '\n\n' + history,
    author="Marcelo F C Gomes, Ivan Ogasawara",
    author_email='marcelo.gomes@fiocruz.br, ivan.ogasawara@gmail.com',
    url='https://github.com/FluVigilanciaBR/fludashboard',
    packages=[
        'fludashboard',
    ],
    package_dir={'fludashboard':
                 'fludashboard'},
    entry_points={
        'console_scripts': [
            'fludashboard=fludashboard.cli:main'
        ]
    },
    include_package_data=True,
    install_requires=requirements,
    license="GNU General Public License v3",
    zip_safe=False,
    keywords='fludashboard',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    test_suite='tests',
    tests_require=test_requirements
)



"""


"""
# local
from .migration import dataset_id as dataset_id_list
from .flu_data import FluDB

import numpy as np
import pandas as pd

contingency_name_from_id = {
    1: 'Nível basal',
    2: 'Nível 0',
    3: 'Nível 1',
    4: 'Nível 2',
}

db = FluDB()


def get_season_level(se):
    """

    :param se: pd.Series
    :return: int
    """
    _max = max([
        se.very_high_level, se.high_level, se.epidemic_level, se.low_level
    ])

    return (
        0 if np.isnan(_max) else
        1 if se.low_level == _max else
        2 if se.epidemic_level == _max else
        3 if se.high_level == _max else
        4
    )


def calc_alert_rank_whole_year(se):
    """
    calculate the alert rank for the whole year

    :param se: pd.Series
    :return:
    """
    high_threshold = se.very_high_level + se.high_level

    return (
        4 if high_threshold >= 5 else
        3 if high_threshold >= 1 else
        2 if se.epidemic_level >= 1 else
        1
    )


def apply_filter_alert_by_epiweek(
    df: pd.DataFrame, view_name: str, epiweek: int=None
):
    """

    :param df:
    :param view_name:
    :param epiweek:
    :return:
    """
    if epiweek is not None:
        mask = df.eval('epiweek=={}'.format(epiweek))
    else:
        mask = df.keys()

    df_alert = df[mask].copy().reset_index()

    return df_alert


def prepare_contingency_level(df: pd.DataFrame):
    epiyear = df.epiyear.max()
    territories_id = df.territory_id.unique()
    alerts = {}

    for territory_id in territories_id:
        alerts[territory_id] = contingency_level(epiyear, territory_id)

    return alerts


def get_contingency_level(se):
    """

    :param se: pd.Series
    :return: int
    """
    return contingency_level(se.epiyear, se.territory_id)


def show_contingency_alert(dataset_id: int, year: int, territory_id: int):
    """

    :param dataset_id:
    :param year:
    :param territory_id:
    :return:
    """
    df = db.get_data(
        dataset_id=dataset_id, scale_id=1, year=year,
        territory_id=territory_id
    )

    # If not obitoflu dataset (3), uses last 4 weeks, o.w. use 3:
    if dataset_id < 3:
        wdw = 4
    else:
        wdw = 3

    weeks = len(df)
    if weeks < wdw+1:
        alert_zone = False
        data_increase = False
    else:
        for i in range(wdw+1, weeks+1):
            alert_zone = any(df.estimated_cases[(i-wdw):i] > df.typical_high[(i-wdw):i])
            data_increase = all(
                df.estimated_cases[(i-wdw):i].values -
                df.estimated_cases[(i-wdw - 1):(i-1)].values > 0
            )
            if (alert_zone & data_increase):
                break

    dataset_from_id = dict(
        zip(dataset_id_list.values(), dataset_id_list.keys())
    )

    print('''
    Data: %s
    Entered alert zone? %s
    Steady increase in the window of interest? %s
    Trigger alert? %s
    ''' % (dataset_from_id[dataset_id], alert_zone, data_increase,
       alert_zone & data_increase)
    )

    return alert_zone & data_increase


def alert_trigger(dataset_id: int, year: int, territory_id: int):
    """

    :param dataset_id:
    :param year:
    :param territory_id:
    :return:
    """
    df = db.get_data(
        dataset_id=dataset_id, scale_id=1, year=year,
        territory_id=territory_id
    )

    # If not obitoflu dataset (3), uses last 4 weeks, o.w. use 3:
    if dataset_id < 3:
        wdw = 4
    else:
        wdw = 3

    weeks = df.shape[0]
    if weeks < wdw+1:
        alert_zone = False
        data_increase = False
    else:
        for i in range(wdw+1, weeks+1):
            alert_zone = any(df.estimated_cases[(i-wdw):i] > df.typical_high[(i-wdw):i])
            data_increase = all(
                df.estimated_cases[(i-wdw):i].values -
                df.estimated_cases[(i-wdw - 1):(i-1)].values > 0
            )
            if (alert_zone & data_increase):
                return alert_zone & data_increase

    return alert_zone & data_increase


def contingency_level(year: int, territory_id: int):
    """

    :param year:
    :param territory_id:
    :return:
    """
    if alert_trigger(dataset_id=3, year=year, territory_id=territory_id):
        return 4
    elif alert_trigger(dataset_id=2, year=year, territory_id=territory_id):
        return 3
    elif alert_trigger(dataset_id=1, year=year, territory_id=territory_id):
        return 2
    return 1
